/**
 * DevExtreme (esm/__internal/core/license/license_validation.js)
 * Version: 23.2.2
 * Build date: Fri Nov 10 2023
 *
 * Copyright (c) 2012 - 2023 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
var __rest = this && this.__rest || function(s, e) {
    var t = {};
    for (var p in s) {
        if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) {
            t[p] = s[p]
        }
    }
    if (null != s && "function" === typeof Object.getOwnPropertySymbols) {
        var i = 0;
        for (p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) {
                t[p[i]] = s[p[i]]
            }
        }
    }
    return t
};
import errors from "../../../core/errors";
import {
    version as packageVersion
} from "../../../core/version";
import {
    base64ToBytes
} from "./byte_utils";
import {
    PUBLIC_KEY
} from "./key";
import {
    pad
} from "./pkcs1";
import {
    compareSignatures
} from "./rsa_bigint";
import {
    sha1
} from "./sha1";
var TokenKind;
! function(TokenKind) {
    TokenKind.corrupted = "corrupted";
    TokenKind.verified = "verified"
}(TokenKind || (TokenKind = {}));
var SPLITTER = ".";
var FORMAT = 1;
var RTM_MIN_PATCH_VERSION = 3;
var GENERAL_ERROR = {
    kind: TokenKind.corrupted,
    error: "general"
};
var VERIFICATION_ERROR = {
    kind: TokenKind.corrupted,
    error: "verification"
};
var DECODING_ERROR = {
    kind: TokenKind.corrupted,
    error: "decoding"
};
var DESERIALIZATION_ERROR = {
    kind: TokenKind.corrupted,
    error: "deserialization"
};
var PAYLOAD_ERROR = {
    kind: TokenKind.corrupted,
    error: "payload"
};
var VERSION_ERROR = {
    kind: TokenKind.corrupted,
    error: "version"
};
var validationPerformed = false;

function verifySignature(_ref) {
    var {
        text: text,
        signature: encodedSignature
    } = _ref;
    return compareSignatures({
        key: PUBLIC_KEY,
        signature: base64ToBytes(encodedSignature),
        actual: pad(sha1(text))
    })
}
export function parseLicenseKey(encodedKey) {
    if (void 0 === encodedKey) {
        return GENERAL_ERROR
    }
    var parts = encodedKey.split(SPLITTER);
    if (2 !== parts.length || 0 === parts[0].length || 0 === parts[1].length) {
        return GENERAL_ERROR
    }
    if (!verifySignature({
            text: parts[0],
            signature: parts[1]
        })) {
        return VERIFICATION_ERROR
    }
    var decodedPayload = "";
    try {
        decodedPayload = atob(parts[0])
    } catch (_a) {
        return DECODING_ERROR
    }
    var payload = {};
    try {
        payload = JSON.parse(decodedPayload)
    } catch (_b) {
        return DESERIALIZATION_ERROR
    }
    var {
        customerId: customerId,
        maxVersionAllowed: maxVersionAllowed,
        format: format
    } = payload, rest = __rest(payload, ["customerId", "maxVersionAllowed", "format"]);
    if (void 0 === customerId || void 0 === maxVersionAllowed || void 0 === format) {
        return PAYLOAD_ERROR
    }
    if (format !== FORMAT) {
        return VERSION_ERROR
    }
    return {
        kind: TokenKind.verified,
        payload: _extends({
            customerId: customerId,
            maxVersionAllowed: maxVersionAllowed
        }, rest)
    }
}
export function validateLicense(licenseKey) {
    var version = arguments.length > 1 && void 0 !== arguments[1] ? arguments[1] : packageVersion;
    if (validationPerformed) {
        return
    }
    validationPerformed = true;
    var warning;
    try {
        var [major, minor, patch] = version.split(".").map(Number);
        var preview = isNaN(patch) || patch < RTM_MIN_PATCH_VERSION;
        if (preview) {
            warning = "W0022";
            return
        }
        if (!licenseKey) {
            warning = "W0019";
            return
        }
        var license = parseLicenseKey(licenseKey);
        if (license.kind === TokenKind.corrupted) {
            warning = "W0021";
            return
        }
        if (!(major && minor)) {
            warning = "W0021";
            return
        }
        if (10 * major + minor > license.payload.maxVersionAllowed) {
            warning = "W0020"
        }
    } catch (e) {
        warning = "W0021"
    } finally {
        if (warning) {
            errors.log(warning)
        }
    }
}
export function setLicenseCheckSkipCondition() {}
export default {
    validateLicense: validateLicense
};
